﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using VehicleRental.Models;
using VehicleRental.WpfClient.Behavior;
using VehicleRental.WpfClient.Data;
using VehicleRental.WpfClient.Helpers;

namespace VehicleRental.WpfClient.ViewModels
{
    public class AppViewModel : ViewModelBase
    {
        private ICommand changeViewModelCommand;
        private IPageViewModel currentViewModel;
        private bool loggedInUser = false;
        private bool anonymousUser = true;
        private ICommand loginCommand;
        private ICommand logoutCommand;

        public IPageViewModel CurrentViewModel
        {
            get
            {
                return this.currentViewModel;
            }
            set
            {
                this.currentViewModel = value;
                this.OnPropertyChanged("CurrentViewModel");
            }
        }

        public bool LoggedInUser
        {
            get
            {
                return this.loggedInUser;
            }
            set
            {
                this.loggedInUser = value;
                this.OnPropertyChanged("LoggedInUser");
            }
        }

        public bool AnonymousUser
        {
            get
            {
                return this.anonymousUser;
            }
            set
            {
                this.anonymousUser = value;
                this.OnPropertyChanged("AnonymousUser");
            }
        }

        public PublicVehicleRentalVIewModel PublicPlacesVM { get; set; }

        public LoginRegisterFormViewModel LoginRegisterVM { get; set; }

        public PrivateVehicleRentalViewModel PrivatePlacesVM { get; set; }

        public List<IPageViewModel> ViewModels { get; set; }

        public ICommand ChangeViewModel
        {
            get
            {
                if (this.changeViewModelCommand == null)
                {
                    this.changeViewModelCommand =
                        new RelayCommand(this.HandleChangeViewModelCommand);
                }
                return this.changeViewModelCommand;
            }
        }

        public ICommand Login
        {
            get
            {
                if (this.loginCommand == null)
                {
                    this.loginCommand = new RelayCommand(this.HandleLoginCommand);
                }
                return this.loginCommand;
            }
        }

        public ICommand Logout
        {
            get
            {
                if (this.logoutCommand == null)
                {
                    this.logoutCommand = new RelayCommand(this.HandleLogoutCommand);
                }
                return this.logoutCommand;
            }
        }

        private void HandleLoginCommand(object parameter)
        {
            this.AnonymousUser = false;
            this.HandleChangeViewModelCommand(this.LoginRegisterVM);
        }

        private void HandleLogoutCommand(object parameter)
        {
            bool isUserLoggedOut = DataPersister.LogoutUser();
            if (isUserLoggedOut)
            {
                this.Username = "";
                this.LoggedInUser = false;
                this.AnonymousUser = true;
                this.HandleChangeViewModelCommand(this.PublicPlacesVM);
            }

        }

        private void HandleChangeViewModelCommand(object parameter)
        {
            var newCurrentViewModel = parameter as IPageViewModel;
            this.CurrentViewModel = newCurrentViewModel;
        }

        public AppViewModel()
        {
            this.ViewModels = new List<IPageViewModel>();
            var loginVM = new LoginRegisterFormViewModel();
            var publicPLacesVM = new PublicVehicleRentalVIewModel();
            var privatePlacesVM = new PrivateVehicleRentalViewModel();
            loginVM.LoginSuccess += this.LoginSuccessful;

            this.PublicPlacesVM = publicPLacesVM;
            this.PrivatePlacesVM = privatePlacesVM;
            this.LoginRegisterVM = loginVM;
            this.CurrentViewModel = this.PublicPlacesVM;
        }

        public void LoginSuccessful(object sender, LoginSuccessArgs e)
        {
            this.Username = e.Username;
            this.LoggedInUser = true;
            this.HandleChangeViewModelCommand(this.PrivatePlacesVM);
        }

        public void CurrentVehicleSuccessful(object sender)
        {
            this.HandleChangeViewModelCommand(this.CurrentVehicleVM);
        }

        public string Username { get; set; }

        public object CurrentVehicleVM { get; set; }
    }
}